@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: mplayer.pl,v 1.28 2006/04/27 16:34:05 khlut Exp $
+# $Id: mplayer.pl,v 1.36 2006/09/19 09:59:42 khlut Exp $
 # Bind mplayer and mencoder with HTML object
 # ElphelOgm is used to receive and record rtp mjpeg video
 #
@@ -25,25 +25,26 @@ use IPC::Open2;
 use threads;
 use threads::shared;
 use POSIX ":sys_wait_h";
-use Gtk2;
+use Gtk2 -init;
 
 ## Parameters from the config file ##
 $conffile="$ENV{HOME}/.mozilla/genres/mplayer.conf";
+$filehelp=
+     "%@  in the filename will be replaced to source URL\n"
+    ."%Y - year \t%m - month number\t%d - day of month\n"
+    ."%H - hour \t%M - minute  \t%S - second\n"
+    ."\$n - number of frames in the file\n"
+    ."\$t - timestamp of the last frame";
 %cfg=(
   vo=>[x11, 'text', "Video output driver (see mplayer -vo help)"],
   ao=>["esd,arts,oss,alsa", 'text', "Audio output driver (see mplayer -ao help)"],
   not_ask=> [0, 'checkbox', "Never show file save dialog again."],
-  out_file=>['%@-%Y-%m-%d_%H-%M-%S.avi', 'file', "Write video to file(s)",
-     "%@  in the filename will be replaced to source URL\n"
-    ."%Y - year \t%m - month number\t%d - day of month\n"
-    ."%H - hour \t%M - minute  \t%S - second"
-  ],
-  out_file_once=>['%@-%Y-%m-%d_%H-%M-%S.avi', 'file', "Save video to file",
-     "%@  in the filename will be replaced to source URL\n"
-    ."%Y - year \t%m - month number\t%d - day of month\n"
-    ."%H - hour \t%M - minute  \t%S - second"
-  ],
-  frames=>  [10000, 'text', "Frames per file limit"],
+  not_ask_jpegs=> [0, 'checkbox', "Never show save as jpegs dialog again."],
+  out_file_once=>['%@-%Y-%m-%d_%H-%M-%S.avi', 'file', "Save this video to file", $filehelp],
+  out_file=>     ['%@-%Y-%m-%d_%H-%M-%S.avi', 'file', "Write video to file(s)", $filehelp],
+  out_file_jpegs=>     ['%@-%Y-%m-%d_%H-%M-\$t.jpeg', 'file', "Save this video to jpeg files", $filehelp],
+  out_file_jpegs_auto=>['%@-%Y-%m-%d_%H-%M-\$t.jpeg', 'file', "Write video to jpeg files", $filehelp],
+  frames=> [10000, 'text', "Frames per file limit"],
 );
 $_=$0;  s|[^/]*$||;  @INC=( "$_../..",@INC);
 require "config.pl";
@@ -52,6 +53,8 @@ load_config();
 
 $v{quittimeout}=2; #wait 2 seconds after send quit command or interruptable signal
 $v{play}=1;
+$v{flip}=0;
+$v{mirror}=0;
 $loop=1;
 $href="/dev/fd/3";
 # output window
@@ -76,11 +79,7 @@ sub gettimeout {
     return localnet() ? 10 : 100;
 }
 sub progress {
-    use Gtk2 ;
-    return unless $v{xid} ;
-    my $W=Gtk2::Gdk::Window->foreign_new(hex $v{xid});
-    my (undef,undef,$w,$h) = $W->get_geometry;
-    my $gc=Gtk2::Gdk::GC->new($W);
+    return unless $W ;
     $W->draw_rectangle ($gc, 1 , 0, 0, $w*$_[0]/50, 10);
     Gtk2::Gdk->flush;
 }
@@ -108,6 +107,7 @@ sub normal_exit{
     $SIG{PIPE}='IGNORE';
     lock $queue,$shutstate;
     alarm ($shutstate? $v{'quittimeout'} : gettimeout);
+    print "\nDESTROY\n";
     close(PIPE_IN);
     close(PIPE_OUT);
     waitpid $slvp, 0;
@@ -129,7 +129,13 @@ $SIG{PIPE}=\&normal_exit;
 $SIG{ALRM}=\&shut;
 
 sub url2mplayer {
-    $v{href}=$v{baseURI}.'/'.$v{href}	if( $v{href}!~m|[a-zA-Z0-9]+:| );
+    if( $v{href} =~ m|^/| ) {
+	if($v{baseURI} !~ m|^file://| && $v{baseURI} =~ m|^[a-zA-Z0-9]+://[^/]/|) {
+	    $v{href}=$&.$v{href};
+	}
+    }else{
+	$v{href}=$v{baseURI}.'/'.$v{href}	if( $v{href}!~m|[a-zA-Z0-9]+:| );
+    }
     $href=$v{href};
     if($ENV{GENRES_MOZILLA_STREAMS} && $v{src} && $href=~m!^(http.?|ftp.?)://!) {
 	$href="/dev/fd/3";
@@ -161,12 +167,25 @@ sub cmd {
     	    lock $queue;
     	    $queue.="pause=$val\nplay=".(1-$val)."\n";
 	}	    
+    }elsif( $var eq "play" ) {
+        $val=0+$val;
+        $val=1 if $val!=0 && $val!=1 ;
+        $val=1-$val;
+        if($val != $pausestate) {
+    	    $pausestate=$val;
+    	    print SLVW "pause\n";
+    	    lock $queue;
+    	    $queue.="pause=$val\nplay=".(1-$val)."\n";
+	}	    
     }elsif( $var eq "filepos" ) {
-        if($val =~ s/%//) {
+	if($val =~ /^[-+]/) {
+    	    print SLVW "seek $val 0\n";	
+        }elsif($val =~ s/%//) {
     	    print SLVW "seek $val 1\n";
         }else{
 	    print SLVW "seek $val 2\n";
         }
+        $pausestate=0;
     }elsif( $var eq "framestep" ) {
         print SLVW "frame_step\n";	    
         $pausestate=1;
@@ -184,7 +203,7 @@ sub cmd {
 	    $queue.="DESTROY\n"; # destroy old stream
 	}
 	if($href eq '/dev/fd/3') {
-	    $queue.="OPEN\n"; # destroy old stream	    
+	    $queue.="OPEN\n"; # open new stream	    
 	}
     }else{
         $v{$var}=$val;
@@ -198,7 +217,7 @@ sub ifilt{
     cmd("filepos=$v{filepos}")      if $v{filepos};
     cmd("framestep=$v{framestep}")  if $v{framestep};
     while(<STDIN>){
-	cmd $_;
+	cmd($_);
     }
 }
 
@@ -291,10 +310,20 @@ print "debug=1\n", # list of control variables
       "autostart=\n",
       "play=1\n",
       "save=\n",
+      "saveformat=\n",
       "fps=\n",
       "lowres=\n",
+      "rotate=0\n",
+      "flip=0\n",
+      "mirror=0\n",
+      "framedelay=0\n",
       "src=\n",
-      "href=\n" unless($ARGV[0] eq '--no-vars');
+      "href=\n",
+      "{this.Play=function(){ this.play=1; }}\n",
+      "{this.Stop=function(){ this.pause=1; }}\n",
+      "{this.Rewind=function(){ this.filepos=0; this.pause=1; }}\n", 
+      "{this.GetURL=function(){ return this.src; }}\n", 
+      "{this.SetURL=function(url){ this.src=url; }}\n" unless($ARGV[0] eq '--no-vars');
       
 while (<STDIN>){
     s/\n//;
@@ -319,26 +348,49 @@ while (<STDIN>){
 	$OPTS="";
 	$v{baseURI}=~s|/[^/]*$||;
 	url2mplayer();
-	if($v{save}) { #Recording TODO: ask about overwriting file
+	if($href ne '/dev/fd/3') {
+	    print "DESTROY\n"; # close unused stream
+	}
+	if($v{save}) { #Recording TODO: ask about file overwriting
 	    $cmd=rtpcmd();
 	    if($cmd) {
-		$cfg{out_file_once}[0]=~s/\.avi/\.ogm/;
-		$cfg{out_file}[0]=~s/\.avi/\.ogm/;
+		$v{saveformat}="ogm" if($v{saveformat} ne "jpeg");
 	    }else{
-		$cfg{out_file_once}[0]=~s/\.ogm/\.avi/;
-		$cfg{out_file}[0]=~s/\.ogm/\.avi/;
+		$v{saveformat}="avi";
 	    }
-	    if(uc $v{save} ne 'AUTO'){
-		if(!file_dialog(out_file_once, frames)) { print "run=0\n"; last};
-		$out_file=$cfg{out_file_once}[0];
-	    }elsif( $cfg{not_ask}[0] ){
-		$out_file=$cfg{out_file}[0];
+	    
+	    if($v{saveformat} eq "jpeg") {
+		$cmd.=" -r -F1" if $cmd;
+	    	if(lc $v{save} ne 'auto'){
+		    if(!file_dialog(out_file_jpegs)) { print "run=0\n"; last};
+		    $out_file=$cfg{out_file_jpegs}[0];
+		}elsif( $cfg{not_ask}[0] ){
+		    $out_file=$cfg{out_file_jpegs_auto}[0];
+		}else{
+		    if(!file_dialog(out_file_jpegs_auto, not_ask_jpegs)) { print "run=0\n"; last};
+		    $out_file=$cfg{out_file_jpegs_auto}[0];
+		}
 	    }else{
-		if(!file_dialog(out_file, not_ask, frames)) { print "run=0\n"; last};
-		$out_file=$cfg{out_file}[0];
+		if($v{saveformat} eq "avi") {
+	    	    $cfg{out_file_once}[0]=~s/\.ogm/\.avi/;
+	    	    $cfg{out_file}[0]=~s/\.ogm/\.avi/;		    
+		}else{
+	    	    $cfg{out_file_once}[0]=~s/\.avi/\.ogm/;
+	    	    $cfg{out_file}[0]=~s/\.avi/\.ogm/;
+		}
+	    	if(lc $v{save} ne 'auto'){
+		    if(!file_dialog(out_file_once, frames)) { print "run=0\n"; last};
+		    $out_file=$cfg{out_file_once}[0];
+		}elsif( $cfg{not_ask}[0] ){
+		    $out_file=$cfg{out_file}[0];
+		}else{
+		    if(!file_dialog(out_file, not_ask, frames)) { print "run=0\n"; last};
+		    $out_file=$cfg{out_file}[0];
+		}
+		$cmd.=" -F '$cfg{frames}[0]'" if $cmd;
 	    }
 	    if($cmd) {
-		$cmd="$cmd -F '$cfg{frames}[0]' -o".suburl($out_file);
+		$cmd.=" -o".suburl($out_file);
 		print "cmd=$cmd\n";
 		runslv $cmd;
 	    } else {
@@ -348,20 +400,42 @@ while (<STDIN>){
 	}else{
 	    if( $v{xid} ) {
 		$OPTS.=" -wid 0x$v{xid}";
+                $W = Gtk2::Gdk::Window->foreign_new(hex $v{xid});
+		
 		if( !$v{width} || $v{width}=~"%" ) {
-		    use Gtk2 ; #-init;
-		    (undef,undef,$v{width},$v{height}) = Gtk2::Gdk::Window->foreign_new(hex $v{xid})->get_geometry;
+		    (undef,undef,$v{width},$v{height}) = $W->get_geometry;
 		}
+                $w=$v{width};
+                $gc=Gtk2::Gdk::GC->new($W);
+                $gc->set_rgb_fg_color( Gtk2::Gdk::Color->new( 
+			(defined $ENV{GENRES_BACKGROUND} && $ENV{GENRES_BACKGROUND} eq "0")? (65535,65535,65535):(0,0,0)
+		) );
 	    }
+	    $p=0;
+	    if( $v{rotate} eq 90 ) { $p=5 }
+	    elsif( $v{rotate} eq 180 ) { $p=3 }
+	    elsif( $v{rotate} eq 270 ) { $p=6 }
+	    $p ^= 2 if( $v{flip} !~ m/false|no|0/i );
+	    $p ^= 1 if( $v{mirror} !~ m/false|no|0/i );
+	    if($p&4) {
+		$FILTERS.=",rotate=".($p&3);
+	    }else{
+		$FILTERS.=",mirror" if( $p&1 );
+		$FILTERS.=",flip" if( $p&2 );
+	    }
+	    print "filters=$FILTERS\n";
+	    $OPTS.=" -vf ".substr($FILTERS,1) if( $FILTERS );
 	    $OPTS.=" -xy '$v{width}'"	if( $v{width} && $v{width}!~"%" );
 	    $OPTS.=" -lavdopts lowres='$v{lowres}'" if( $v{lowres} );
 	    $OPTS.=" -fps '$v{fps}'"		if( $v{fps} );
-	    $OPTS.=" -fixed-vo -loop ".(0+$v{loop}) if( $href ne "/dev/fd/3" && exists $v{loop} && "\L$v{loop}" ne 'false' );
-	    $loop= 0+$v{loop} if( exists $v{loop} && "\L$v{loop}" ne 'false' );
+	    $OPTS.=" -fixed-vo -loop ".(0+$v{loop}) if( $href ne "/dev/fd/3" && exists $v{loop} && lc $v{loop} ne 'false' );
+	    $loop= 0+$v{loop} if( exists $v{loop} && lc $v{loop} ne 'false' );
 	    $OPTS.= localnet()? " -nocache" : " -cache 1000 -cache-min 50";
 	    if(( $cmd2=rtpcmd() )) {
+	        $cmd2.=" -i -q";
 		$cmd2.=" -k $v{frameskip}" if $v{frameskip};
-	        $cmd2="|exec $cmd2 -i 2>/dev/null ".fileno(PIPE_OUT)."<&- >/dev/fd/".fileno(PIPE_IN);
+		$cmd2.=" -b $v{framedelay}" if $v{framedelay};
+	        $cmd2="|exec $cmd2 2>/dev/null ".fileno(PIPE_OUT)."<&- >/dev/fd/".fileno(PIPE_IN);
 		$cmd="mplayer $OPTS -vo $cfg{vo}[0] -zoom -nojoystick -slave -nomouseinput -cookies /dev/fd/".fileno(PIPE_OUT);
 	        print "cmd=$cmd\n";
 	        print "cmd2=$cmd2\n";
